@@ -1,6 +1,6 @@
 package Mail::DMARC;
 {
-  $Mail::DMARC::VERSION = '0.20130510';
+  $Mail::DMARC::VERSION = '0.20130514';
 }
 use strict;
 use warnings;
@@ -9,7 +9,6 @@ use Carp;
 
 require Mail::DMARC::DNS;
 require Mail::DMARC::Policy;
-require Mail::DMARC::PurePerl;
 require Mail::DMARC::Report;
 require Mail::DMARC::Result;
 
@@ -60,16 +59,7 @@ sub dkim {
     return $self->{dkim} if ! $dkim;
 
     if ( ref $dkim && ref $dkim eq 'Mail::DKIM::Verifier' ) {
-# A DKIM verifier will have result and signature methods.
-        foreach my $s ( $dkim->signatures ) {
-            push @{ $self->{dkim} }, {
-                domain       => $s->domain,
-                selector     => $s->selector,
-                result       => $s->result,
-                human_result => $s->result_detail,
-            };
-        };
-        return $self->{dkim};
+        return $self->dkim_from_mail_dkim($dkim);
     };
 
     if ( 'ARRAY' ne ref $dkim ) {
@@ -77,14 +67,26 @@ sub dkim {
     };
     foreach my $s ( @$dkim ) {
         foreach my $f ( qw/ domain result / ) {
-            if ( ! $s->{$f} ) {
-                croak "DKIM '$f' is required!";
-            };
+            croak "DKIM '$f' is required!" if ! $s->{$f};
         };
     };
     return $self->{dkim} = $dkim;
 };
 
+sub dkim_from_mail_dkim {
+    my ($self, $dkim) = @_;
+# A DKIM verifier will have result and signature methods.
+    foreach my $s ( $dkim->signatures ) {
+        push @{ $self->{dkim} }, {
+            domain       => $s->domain,
+            selector     => $s->selector,
+            result       => $s->result,
+            human_result => $s->result_detail,
+        };
+    };
+    return $self->{dkim};
+};
+
 sub spf {
     my ($self, @args) = @_;
     return $self->{spf} if 0 == scalar @args;
@@ -114,7 +116,7 @@ sub policy {
 sub report {
     my $self = shift;
     return $self->{report} if ref $self->{report};
-    return $self->{report} = Mail::DMARC::Report->new( $self );
+    return $self->{report} = Mail::DMARC::Report->new();
 };
 
 sub result {
@@ -157,7 +159,7 @@ Mail::DMARC - Perl implementation of DMARC
 
 =head1 VERSION
 
-version 0.20130510
+version 0.20130514
 
 =head1 SYNOPSIS
 
@@ -185,8 +187,6 @@ L<Mail::DMARC::Policy> - a DMARC record in object format
 
 L<Mail::DMARC::PurePerl> - a DMARC implementation
 
-L<Mail::DMARC::Report>
-
 =over 4
 
 =item L<Mail::DMARC::Result>
@@ -195,6 +195,20 @@ L<Mail::DMARC::Report>
 
 =back
 
+L<Mail::DMARC::Report> - the R in DMARC
+
+=over 4
+
+L<Mail::DMARC::Report::Send> - deliver formatted reports via SMTP & HTTP
+
+L<Mail::DMARC::Report::Receive> - parse incoming email and HTTP reports to store
+
+L<Mail::DMARC::Report::Store> - a persistent data store for DMARC reports
+
+L<Mail::DMARC::Report::View> - CLI and (eventually) CGI methods for report viewing
+
+=back
+
 L<Mail::DMARC::libopendmarc|http://search.cpan.org/~shari/Mail-DMARC-opendmarc> - an XS implementation using libopendmarc
 
 =head1 HOW TO USE
@@ -219,13 +233,13 @@ L<Mail::DMARC::libopendmarc|http://search.cpan.org/~shari/Mail-DMARC-opendmarc>
         ...continue normal processing...
     };
 
-There's a lot more information available in the $result object. See the L<Mail::DMARC::Result> page for complete details.
+There's a lot more information available in the $result object. See L<Mail::DMARC::Result> page for complete details.
 
 =head1 METHODS
 
 =head2 new
 
-Create a new empty DMARC object. Then populate it and run the request:
+Create an empty DMARC object. Then populate it and run the request:
 
     my $dmarc = Mail::DMARC->new;
     $dmarc->source_ip('192.0.1.1');
