@@ -1,10 +1,11 @@
 package Mail::DMARC::Report::View::CLI;
 {
-  $Mail::DMARC::Report::View::CLI::VERSION = '0.20130510';
+  $Mail::DMARC::Report::View::CLI::VERSION = '0.20130514';
 }
 use strict;
 use warnings;
 
+use Carp;
 use Data::Dumper;
 
 require Mail::DMARC::Report::Store;
@@ -19,10 +20,20 @@ sub list {
     my $reports = $self->store->retrieve;
     foreach my $report ( @$reports ) {
         printf "%3s  %30s  %15s %15s\n", @$report{ qw/ id domain begin end / };
+        foreach my $row ( @{$report->{rows}} ) {
+            printf "\t%15s  %6s  %6s \n", @$row{ qw/ disposition dkim spf / };
+        };
     };
     return $reports;
 };
 
+sub detail {
+    my $self = shift;
+    my $id = shift or croak "need an ID!";
+    return $id;
+};
+
+
 sub store {
     my $self = shift;
     return $self->{store} if ref $self->{store};
@@ -41,7 +52,7 @@ Mail::DMARC::Report::View::CLI - view locally stored DMARC reports
 
 =head1 VERSION
 
-version 0.20130510
+version 0.20130514
 
 =head1 AUTHORS
 
